/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"
#include "ili9341.h"
#include "ili9341.h"                // SYS function prototypes


// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
volatile bool flag = false;

// Callback funckija za eksterni interrupt na pinu RB15 (setovanje flag-a ukoliko se stanje na pinu promeni na logicku 0):
void GPIO_RB15_IntHandler(GPIO_PIN pin, uintptr_t context){
    if(GPIO_PinRead(pin)==0){
        flag = true;
    }
}

// Funkcija za sortiranje niza od najmanje do najvece vrednosti:
void sort(uint16_t* arr, int n){
    int16_t i, key, j;
    
    for(i=1;i<n;i++){
        key = arr[i];
        
        j = i-1;
        
        while(j>=0 && arr[j]>key){
            arr[j+1] = arr[j];
            j=j-1;
        }
        arr[j+1] = key;
    }
}

void readXY(uint16_t* x, uint16_t* y){
    uint16_t data[21];
    
    // Pin RB3 je ulazni pin (Hi-Z):
    GPIO_PinInputEnable(GPIO_PIN_RB3);
    // Pin RB15 je ulazni (Hi-Z) i na njemu je onemoguceno generisanje eksternih interrupt-a:
    GPIO_PinIntDisable(GPIO_PIN_RB15);
    GPIO_RB15_InputEnable();                                
    // Pin RB2 postavlja se kao izlazni pin i na njemu se definise stanje logicke 1 (Vcc):
    GPIO_PinOutputEnable(GPIO_PIN_RB2);
    GPIO_PinSet(GPIO_PIN_RB2);

    for(uint8_t i=0; i<21; i++)                                                 // A/D konverzija se vrsi 21 put, kako bi se na kraju na 21 odbirak primenio median filtar i uklonili potencijalne ekstremne vrednosti usled bouncing-a.
    {
        ADCHS_ChannelConversionStart(ADCHS_CH3);
        while(!ADCHS_ChannelResultIsReady(ADCHS_CH3));
        data[i] = ADCHS_ChannelResultGet(ADCHS_CH3);                            // Ocitavanje izlaza A/D konvertora na ciji se ulaz dovodi signal sa pina RB3.
        CORETIMER_DelayUs(10);
    }
    sort(data, 21);                                                             // Sortiranje odbiraka od najmanjeg do najveceg.
    x[0] = data[10];                                                            // Uzimanje median vrednosti iz niza.

    // Pin RB2 je ulazni pin (Hi-Z):
    GPIO_PinInputEnable(GPIO_PIN_RB2);
    // Pin RB3 postavlja se kao izlazni pin i na njemu se definise stanje logicke 1 (Vcc):
    GPIO_PinOutputEnable(GPIO_PIN_RB3);
    GPIO_PinSet(GPIO_PIN_RB3);
    // Pin RB14 je ulazni pin (Hi-Z):
    GPIO_RB14_InputEnable();
    // Pin RB15 postavlja se kao izlazni pin i na njemu se definise stanje logicke 0 (GND):
    GPIO_RB15_OutputEnable();
    GPIO_RB15_Clear();

    for(uint8_t i=0; i<21; i++)                                                 
    {
        ADCHS_ChannelConversionStart(ADCHS_CH2);
        while(!ADCHS_ChannelResultIsReady(ADCHS_CH2));
        data[i] = ADCHS_ChannelResultGet(ADCHS_CH2);                            // Ocitavanje izlaza A/D konvertora na ciji se ulaz dovodi signal sa pina RB2.
        CORETIMER_DelayUs(10);
    }
    sort(data, 21);
    y[0] = data[10];

    // Standby mod:
    GPIO_RB14_OutputEnable();
    GPIO_RB14_Clear();
    GPIO_PinInputEnable(GPIO_PIN_RB2);
    GPIO_RB15_InputEnable();
    GPIO_PinInputEnable(GPIO_PIN_RB3);
    CORETIMER_DelayMs(1);
    GPIO_PinIntEnable(GPIO_PIN_RB15, GPIO_INTERRUPT_ON_MISMATCH);

}

// Racunanje koeficijenata A, B, C, D, E i F za kalibraciju u tri tacke P1, P2 i P3:
void calculateCoeffs(float* coeff, uint8_t* xdyd, uint16_t* xtyt){
    coeff[0] = (float)(xdyd[0]*(xtyt[3]-xtyt[5])+xdyd[2]*(xtyt[5]-xtyt[1])+xdyd[4]*(xtyt[1]-xtyt[3]))/(float)(xtyt[0]*(xtyt[3]-xtyt[5])+xtyt[2]*(xtyt[5]-xtyt[1])+xtyt[4]*(xtyt[1]-xtyt[3]));
    coeff[1] = (float)(coeff[0]*(xtyt[4]-xtyt[2])+xdyd[2]-xdyd[4])/(float)(xtyt[3]-xtyt[5]);
    coeff[2] = (float)(xdyd[4]-coeff[0]*xtyt[4]-coeff[1]*xtyt[5]); 
    coeff[3] = (float)(xdyd[1]*(xtyt[3]-xtyt[5])+xdyd[3]*(xtyt[5]-xtyt[1])+xdyd[5]*(xtyt[1]-xtyt[3]))/(float)(xtyt[0]*(xtyt[3]-xtyt[5])+xtyt[2]*(xtyt[5]-xtyt[1])+xtyt[4]*(xtyt[1]-xtyt[3]));
    coeff[4] = (float)(coeff[3]*(xtyt[4]-xtyt[2])+xdyd[3]-xdyd[5])/(float)(xtyt[3]-xtyt[5]);
    coeff[5] = (float)(xdyd[5] - coeff[3]*xtyt[4] - coeff[4]*xtyt[5]);
}

int main ( void )
{
    /* Initialize all modules */
    SYS_Initialize ( NULL );

    
    ili9341_init();                                                             // Inicijalizacija displeja.
    ili9341_clear_display(COLOR_WHITE);                                         // Brisanje sadrzaja displeja.
    
    // Pin RB14 postavlja se kao izlazni pin i na njemu se definise stanje logicke 0 (GND):
    GPIO_RB14_Clear();
    GPIO_RB14_OutputEnable();
    
    GPIO_PinInterruptCallbackRegister(GPIO_PIN_RB15, GPIO_RB15_IntHandler, 0);  // Registrovanje GPIO_RB15_IntHandler funkcije kao callback funkcije za eksterni interrupt na pinu RB15.
    GPIO_PinIntEnable(GPIO_PIN_RB15, GPIO_INTERRUPT_ON_MISMATCH);               // Omogucavanje generisanja interrupt-a na pinu RB15 za svaku promenu stanja na pinu (ON MISMATCH).   
    
    uint8_t cnt=0;
    uint8_t xdyd[6]={32, 120, 160, 24, 32, 24};                                 // Poznate koordinate dispeleja tri tacke na displeju u kojima se vrsi kalibracija (P1, P2 i P3).
    uint16_t xtyt[6];
    uint16_t xt[1], yt[1];
    float coeff[6];
    printf("START CALIB\r\n");
    
    while(cnt<3){                                                               // Postupak ocitavanja pritiska touch panela u zadatoj tacki ponavlja se tri puta za kalibracione tacke P1, P2 i P3.
        ili9341_draw_filled_circle(xdyd[2*cnt],xdyd[2*cnt+1],7,COLOR_RED);      // Prikaz tacke Pi, i = 1,2,3.
        if(flag){                                                               // Ukoliko se desio interrupt na RB15:    
            flag = false;                                                       // Resetovanje flag-a.
            readXY(xt, yt);                                                     // Ocitavanje koordinata touch panela.
            printf("Tacka %u\r\n", cnt);
            xtyt[2*cnt] = xt[0];                                                // Cuvanje koordinata touch panela za tacke P1, P2 i P3 u nizu xtyt ---> {xP1, yP1, xP2, yP2, xP3, yP3}.
            xtyt[2*cnt+1] = yt[0];
            
            cnt++;
            CORETIMER_DelayMs(2000);
        }   
        
    }
    
    ili9341_clear_display(COLOR_WHITE);                                         // Brisanje sadrzaja dispeleja.
    
    calculateCoeffs(coeff, xdyd, xtyt);                                         // Racunanje kalibracionih koeficijenata na osnovu ocitavanja koordinata sa touch panela i poznatih koordinata displeja.
    printf("P1: (%u, %u)|P2: (%u, %u)|P3: (%u, %u) \r\n", xtyt[0], xtyt[1], xtyt[2], xtyt[3], xtyt[4], xtyt[5]);
    printf("A: %.3f|B: %.3f|C: %.3f\r\n", coeff[0], coeff[1], coeff[2]);
    printf("D: %.3f|E: %.3f|F: %.3f\r\n", coeff[3], coeff[4], coeff[5]);
    
    while ( true ){

        if(flag){                                                               // Ukoliko se desio interrupt na RB15:    
            flag = false;                                                       // Resetovanje flag-a.
            ili9341_clear_display(COLOR_WHITE);                                 // Brisanje sadrzaja dispeleja.
            
            uint16_t x[1];
            uint16_t y[1];

            readXY(x,y);                                                        // Ocitavanje koordinata touch panela.
            x[0] = (uint16_t)(coeff[0]*x[0]+coeff[1]*y[0]+coeff[2]);            // Preracunavanje ocitanih koordinata u koordinate displeja na osnovu kalibracionih koeficijenata A, B, C, D, E i F.
            y[0] = (uint16_t)(coeff[3]*x[0]+coeff[4]*y[0]+coeff[5]);
            
            printf("Koordinate: %u x %u\r\n", x[0],y[0]);
            if(x[0]<320 && y[0]<240){                                           // Za koordinate unutar dispelj 320x240:
                ili9341_draw_filled_circle(x[0],y[0],15,COLOR_CYAN);            // Crtanje kruga sa centrom u tacki pritiska touch panela.
            }
            
        }
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

